// +build !ignore_autogenerated

/*
Copyright 2017 The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// This file was autogenerated by deepcopy-gen. Do not edit it manually!

package interfaces

import (
	conversion "k8s.io/apimachinery/pkg/conversion"
	reflect "reflect"
)

// GetGeneratedDeepCopyFuncs returns the generated funcs, since we aren't registering them.
func GetGeneratedDeepCopyFuncs() []conversion.GeneratedDeepCopyFunc {
	return []conversion.GeneratedDeepCopyFunc{
		{Fn: DeepCopy_interfaces_Ttest, InType: reflect.TypeOf(&Ttest{})},
	}
}

// DeepCopy_interfaces_Ttest is an autogenerated deepcopy function.
func DeepCopy_interfaces_Ttest(in interface{}, out interface{}, c *conversion.Cloner) error {
	{
		in := in.(*Ttest)
		out := out.(*Ttest)
		*out = *in
		if in.I != nil {
			in, out := &in.I, &out.I
			*out = make([]Inner, len(*in))
			for i := range *in {
				if newVal, err := c.DeepCopy(&(*in)[i]); err != nil {
					return err
				} else {
					(*out)[i] = *newVal.(*Inner)
				}
			}
		}
		return nil
	}
}
